#ifndef __AUDIO_PLAYER_H__
#define __AUDIO_PLAYER_H__
#include <e32base.h>
class RFs;
class SoundMixer;
class MidiDriver;
class MidiParser;

class CMidiPlayer;
class CStreamPlayer;

class MCompleteListener
    {
    public:
        virtual void completeListen(int aErr, TInt aBytesWritten) = 0;
    };



class CAudioPlayer : public CBase
    {
    public:
        CAudioPlayer();
        void ConstructL();
        ~CAudioPlayer();
        CMidiPlayer& Midi();
        CStreamPlayer& Channel(TInt aChannel);
        TInt ChannelIndex(TInt aChannel) const;
        TInt MaxVolume() const;
        TInt Volume(const TUid& aChannel);
        void SetVolume(TInt aVolume, const TUid&  aChannel);
        void SetPause(TBool aPause, const TUid&  aChannel);
        void SetPause(TBool aPause);
        void StopAll();
        void Stop(const TUid& aChannel);
        TBool IsPlaying(const TUid&  aChannel) const;
    private:
        void completeListen(TInt aErr, TInt aBytesWritten);
//        void channelRead(const PlayingSoundHandle& aHandle, int aAmount);
    private:
        SoundMixer* iMixer;
        CMidiPlayer* iMidiPlayer;
    };



class CMidiPlayer : public CBase
    {
    public:
        CMidiPlayer(SoundMixer& aMixer);
        void ConstructL();
        ~CMidiPlayer();
        void PlayMidiFromFileL(const TDesC& aFileName, TBool aLooping); //from file
        void PlayMidiL(const TDesC8& aMidiData, TBool aCopy, TBool aLooping); //from memory, copies the data
        void PlayMidiL(HBufC8* aData, TBool aLooping); //from memory and takes data ownership
        void SetVolume(TInt aVolume);
        TInt Volume();
        TBool IsPlaying() const;
		TBool IsLoaded() const;
        void Stop();
        void SetPause(TBool aPause);
    private:
        SoundMixer& iMixer;
        MidiDriver* iDriver;
        MidiParser* iParser;
        const TDesC8* iMidiData;
        HBufC8* iMidiDataBuffer;
		TInt iVolume;
    };





#endif

